import os
from typing import TYPE_CHECKING, Any

from celery import Celery

try:
    from plaster_yaml.loader import resolve_use
except ImportError:

    def resolve_use(use: str, entrypoint: str) -> Any:
        raise LookupError("Install celery_yaml[pyramid]")


if TYPE_CHECKING:
    from pyramid import Configurator  # type: ignore


def resolve_entrypoint(use: str) -> Celery:
    entrypoint = "celery_yaml.app"
    app = resolve_use(use, entrypoint)
    if not isinstance(app, Celery):
        raise ValueError(f"{use} does not point to a valid Celery instance.")
    return app


def includeme(config: "Configurator") -> None:
    settings = config.registry.settings
    app = resolve_entrypoint(settings["celery"].pop("use"))
    celeryconf = settings["celery"]
    if "CELERY_BROKER_URL" in os.environ:
        # override the browker url
        celeryconf["broker_url"] = os.environ["CELERY_BROKER_URL"]
    app.config_from_object(celeryconf)



import importlib.metadata

from .loader import add_yaml_option
from .pyramid import includeme

__version__ = importlib.metadata.version("celery-yaml")

__all__ = [
    "add_yaml_option",
    "includeme",
]



"""Helper for celery."""

import logging
import os
import sys
from collections.abc import Mapping
from logging.config import dictConfig
from typing import Any

import celery.loaders.base
import envsub
import yaml
from celery import Celery
from celery.signals import user_preload_options  # type: ignore
from click import Option

log = logging.getLogger(__name__)


@user_preload_options.connect
def on_preload_parsed(options: Mapping[str, Any], **kwargs: Any) -> None:
    config_path = options["yaml"]
    config_key = options["yaml_key"]
    app = kwargs["app"]

    if config_path is None:
        print("You must provide the --yaml argument")
        sys.exit(-1)

    loader = YamlLoader(app, config=config_path, config_key=config_key)
    loader.read_configuration()


def add_yaml_option(app: Celery) -> None:
    help = "Celery configuration in a YAML file."
    help_key = "The key that contains the celery configuration."
    app.user_options["preload"].add(Option(["--yaml"], required=True, help=help))
    app.user_options["preload"].add(
        Option(
            ["--yaml-key"],
            required=False,
            default="celery",
            show_default=True,
            help=help_key,
        )
    )


class YamlLoader(celery.loaders.base.BaseLoader):
    """Celery loader based on yaml file."""

    def __init__(
        self,
        app: Celery,
        config: str,
        *,
        config_key: str = "celery",
        configure_logging: bool = True,
    ) -> None:
        self.app = app
        self.config_path = config
        self.config_key = config_key
        self.configure_logging = configure_logging
        super().__init__(app)

    def read_configuration(
        self, env: str = "CELERY_CONFIG_MODULE"
    ) -> Mapping[str, Any]:
        """Override this method to configure the celery app."""
        with open(self.config_path) as downstream:
            with envsub.sub(downstream) as upstream:
                _conf = yaml.safe_load(upstream)
        _celery_config = _conf[self.config_key]
        if "CELERY_BROKER_URL" in os.environ:
            # override the browker url
            _celery_config["broker_url"] = os.environ["CELERY_BROKER_URL"]

        self.app.config_from_object(_celery_config)
        if self.configure_logging and "logging" in _conf:
            dictConfig(_conf["logging"])

        if hasattr(self.app, "on_yaml_loaded"):
            self.app.on_yaml_loaded(_conf, config_path=self.config_path)
        return _celery_config
